@@ -1,5 +1,10 @@
 #!/usr/bin/perl
 
+package main;
+
+our $config;
+our $options;
+
 package App::Music::ChordPro::Output::Common;
 
 use strict;
@@ -7,6 +12,11 @@ use warnings;
 use App::Music::ChordPro::Chords;
 use String::Interpolate::Named;
 use utf8;
+use POSIX qw(setlocale LC_TIME strftime);
+
+use parent qw(Exporter);
+our @EXPORT;
+our @EXPORT_OK;
 
 sub fmt_subst {
     my ( $s, $t ) = @_;
@@ -21,12 +31,20 @@ sub fmt_subst {
 	        @{$m->{key}} ];
     }
     $m->{tuning} //= [ join(" ", App::Music::ChordPro::Chords::get_tuning) ];
-    $m->{instrument} //= [ $::config->{instrument} ];
-
+    $m->{instrument} = [ $config->{instrument}->{type} ];
+    $m->{"instrument.type"} = [ $config->{instrument}->{type} ];
+    $m->{"instrument.description"} = [ $config->{instrument}->{description} ];
+    $m->{user} = [ $config->{user}->{name} ];
+    $m->{"user.name"} = [ $config->{user}->{name} ];
+    $m->{"user.fullname"} = [ $config->{user}->{fullname} ];
+    setlocale( LC_TIME, "" );
+    $m->{today} //= [ strftime( $config->{dates}->{today}->{format},
+				localtime(time) ) ];
     interpolate( { %$s, args => $m,
-		   separator => $::config->{metadata}->{separator} },
+		   separator => $config->{metadata}->{separator} },
 		 $t );
 }
+push( @EXPORT, 'fmt_subst' );
 
 # Roman - functions for converting between Roman and Arabic numerals
 # 
@@ -51,6 +69,7 @@ sub isroman($) {
                 (?: L?X{0,3} | X[LC])
                 (?: V?I{0,3} | I[VX])$/ix;
 }
+push( @EXPORT_OK, 'isroman' );
 
 sub arabic($) {
     my $arg = shift;
@@ -64,6 +83,7 @@ sub arabic($) {
     }
     $arabic;
 }
+push( @EXPORT_OK, 'arabic' );
 
 sub Roman($) {
     my $arg = shift;
@@ -87,10 +107,12 @@ sub Roman($) {
     }
     $roman;
 }
+push( @EXPORT_OK, 'Roman' );
 
 sub roman($) {
     lc Roman shift;
 }
+push( @EXPORT_OK, 'roman' );
 
 # Prepare outlines.
 # This mainly untangles alternative names when being sorted on.
@@ -197,14 +219,14 @@ sub prep_outlines {
     if ( @{$ctl->{fields}} == 1 ) {
 	@book =
 	  sort { $a->[0] cmp $b->[0] }
-	  map { [ lc($_->{meta}->{$ctl->{fields}->[0]}->[0]), $_ ] }
+	  map { [ demarkup(lc($_->{meta}->{$ctl->{fields}->[0]}->[0])), $_ ] }
 	  @book;
     }
     elsif ( @{$ctl->{fields}} == 2 ) {
 	@book =
 	  sort { $a->[0] cmp $b->[0] || $a->[1] cmp $b->[1] }
-	  map { [ lc($_->{meta}->{$ctl->{fields}->[0]}->[0]),
-		  lc($_->{meta}->{$ctl->{fields}->[1]}->[0]),
+	  map { [ demarkup(lc($_->{meta}->{$ctl->{fields}->[0]}->[0])),
+		  demarkup(lc($_->{meta}->{$ctl->{fields}->[1]}->[0])),
 		  $_ ] }
 	  @book;
     }
@@ -214,5 +236,14 @@ sub prep_outlines {
 
     return \@book;
 }
+push( @EXPORT_OK, 'prep_outlines' );
+
+# Remove markup.
+sub demarkup {
+    my ( $t ) = @_;
+    $t =~ s;</?([-\w]+|span\s.*?)>;;g;
+    return $t;
+}
+push( @EXPORT, 'demarkup' );
 
 1;
